const system = require("../../../system");
var settings = require("../../../../config/settings");
/**
 * 飞书小程序相关接口
 */
class UtilsFeishuService{
  constructor() {
    this.execClient = system.getObject("util.execClient");
    this.cacheManager = system.getObject("db.common.cacheManager");
    this.appConfig={
      app_id:"cli_9e28dcb1d637100d",
      app_secret:"zL1uRrWFzwhFbLWZrmSuCh72JGjJXQg0"
    }
  };
  //订阅验证
  async subscribeVerification(pobj){
    if(pobj && pobj.challenge){
      return system.getResultSuccess(pobj.challenge);
    }else{
      return system.getResultFail();
    }
  }
  //接收app_ticket
  async receiveAppTicket(pobj){
    if(pobj && pobj.event && pobj.event.app_ticket && pobj.event.type && pobj.event.type=="app_ticket"){
      //将app_ticket保存到缓存
      await this.cacheManager["feishuAppTicketCache"].set(pobj.event.app_ticket);
      return system.getResultSuccess();
    }
    return system.getResultFail();
  }
  //重新推送app_ticket（主动触发)
  async resendAppTicket(){
    var url = "https://open.feishu.cn/open-apis/auth/v3/app_ticket/resend";
    var pobj = {
      app_id:this.appConfig.app_id,//应用唯一标识，创建应用后获得
      app_secret:this.appConfig.app_secret//应用秘钥，创建应用后获得
    };
    var rtn = await this.execClient.execPost(pobj, url);
    if (!rtn || !rtn.stdout) {
      return system.getResult(null, "execPost data is empty");
    }
    var result = JSON.parse(rtn.stdout);
    if(result.code==0){
      return system.getResultSuccess();
    }
    return system.getResult(null, "execPost data is empty.错误码："+result.code);
  }
  //获取app_access_token（应用商店应用）
  async getAppAccessToken(){
    var appAccessToken = await this.cacheManager["feishuAppAccessTokenCache"].get();
    if(appAccessToken){
      return system.getResultSuccess(appAccessToken);
    }
    var app_ticket = await this.cacheManager["feishuAppTicketCache"].get();
    if(!app_ticket){
      await this.resendAppTicket();//重新推送app_ticket
      return system.getResultFail("app_ticket不存在");
    }
    var url = "https://open.feishu.cn/open-apis/auth/v3/app_access_token";
    var obj = {
      app_id:this.appConfig.app_id,//应用唯一标识，创建应用后获得
      app_secret:this.appConfig.app_secret,//应用秘钥，创建应用后获得
      app_ticket:app_ticket//平台定时推送给应用的临时凭证，通过事件监听机制获得，详见订阅事件
    };
    var rtn = await this.execClient.execPost(obj, url);
    if (!rtn || !rtn.stdout) {
      return system.getResult(null, "execPost data is empty");
    }
    var result = JSON.parse(rtn.stdout);
    if(result.code==0 && result.app_access_token){
      //缓存AppAccessToken
      await this.cacheManager["feishuAppAccessTokenCache"].set(result.app_access_token);
      return system.getResultSuccess(result.app_access_token);
    }
    return system.getResultFail("获取appAccessToken失败");
  }
  //获取登录用户身份
  async getUserAccessToken(obj){
    var resData = {};
    if(obj.open_id){
      //缓存中获取登录用户身份
      resData = await this.cacheManager["feishuUserAccessTokenCache"].get(obj.open_id);
    }
    //缓存中存在 直接返回
    if(resData && resData.access_token){
      return system.getResultSuccess(resData);
    }
    if(!obj.code){
      return system.getResultFail("code参数不能为空");
    }
    if(!obj.app_access_token){
      return system.getResultFail("app_access_token参数不能为空");
    }
    var url = "https://open.feishu.cn/open-apis/authen/v1/access_token";
    var fsObj={
      app_access_token:obj.app_access_token,//应用的 app_access_token，必须与请求身份验证中的应用保持一致
      grant_type:"authorization_code",//在本流程中，此值为 authorization_code
      code:obj.code //来自请求身份验证(新)流程，用户扫码登录后会自动302到redirect_uri并带上此参数
    };
    var rtn = await this.execClient.execPost(fsObj, url);
    if (!rtn || !rtn.stdout) {
      return system.getResult(null, "execPost data is empty");
    }
    var result = JSON.parse(rtn.stdout);
    if(result.code==0 && result.data){
      //缓存AppAccessToken
      await this.cacheManager["feishuUserAccessTokenCache"].set(result.data,result.data.open_id);
      return system.getResultSuccess(result.data);
    }
    return system.getResult(null, "获取飞书登录用户身份失败");
  }
  //获取用户信息
  async getUserInfo(pobj){
    if(!pobj.user_access_token){
      return system.getResultFail("user_access_token参数不能为空")
    }
    var url = "https://open.feishu.cn/open-apis/authen/v1/user_info";
    var obj = {
      user_access_token:pobj.user_access_token
    };
    var rtn = await this.execClient.execGet(obj, url);
    if (!rtn || !rtn.stdout) {
      return system.getResult(null, "execPost data is empty");
    }
    var result = JSON.parse(rtn.stdout);
    if(result.code==0 && result.data){
      return system.getResultSuccess(result.data);
    }
    return system.getResult(null, "获取飞书用户信息失败");
  }
}
module.exports = UtilsFeishuService;
